package Spreadsheet::ParseXLSX::ChopRPH;
$Spreadsheet::ParseXLSX::ChopRPH::VERSION = '0.201409142';
use strict;
use base qw(Spreadsheet::ParseXLSX);
use Data::Dumper;
use File::Basename qw(dirname);
use File::Spec::Functions qw(catfile);
use PadWalker qw(peek_my);

# 0.14まで
sub _parse_xml {
    my $self = shift;
    my ($zip, $subfile) = @_;

    my $member = $zip->memberNamed($subfile);
    die "no subfile named $subfile" unless $member;

    my $xml = XML::Twig->new;

    #$xml->parse(scalar $member->contents);
    my $content = scalar $member->contents;
    $content =~ s!<rPh[^>]*>.*?</rPh>!!gsm;
    $xml->parse($content);

    return $xml;
}

# 0.15から
sub _parse_shared_strings {
    my $self = shift;
    my ($strings) = @_;

    $strings =~ s!<rPh[^>]*>.*?</rPh>!!gsm;
 
    $self->SUPER::_parse_shared_strings($strings);
}

sub _parse_sheet {
    my $self = shift;
    my ($sheet, $sheet_file) = @_;
    my $rid = $_->att('r:id');
    my $zip = ${peek_my(1)->{'$zip'}};

    $self->SUPER::_parse_sheet($sheet, $sheet_file);

    my $type_base = 'http://schemas.openxmlformats.org/officeDocument/2006/relationships';

    my $rels = $self->_parse_xml( $zip, $self->_rels_for('') );

    my $wb_name = ( $rels->find_nodes(qq<//Relationship[\@Type="$type_base/officeDocument"]>) )[0]->att('Target');
    my $wb_rels = $self->_parse_xml( $zip, $self->_rels_for($wb_name) );

    my $sheet_name = ($wb_rels->find_nodes(qq<//Relationship[\@Type="$type_base/worksheet" and \@Id="$rid"]>))[0]->att('Target');
    my $sheet_path = catfile dirname($wb_name), $sheet_name;
    return unless $zip->memberNamed( $self->_rels_for($sheet_path) );
    my $sheet_rels = $self->_parse_xml( $zip, $self->_rels_for($sheet_path) );

    for my $comment_node ($sheet_rels->find_nodes(qq<//Relationship[\@Type="$type_base/comments"]>)) {
        my $comment_name = $comment_node->att('Target');
        my $comment_path = catfile dirname($sheet_path), $comment_node->att('Target');
        1 while $comment_path =~ s{(^|/)(?!\.\./)[^/]+/\.\./}{$1}; # xl/worksheets/../comments1.xml -> xl/comments1.xml
        my $comment_contents = $zip->memberNamed($comment_path)->contents;
        my $comment_xml = XML::Twig->new( 
            twig_roots => {
                comment => sub {
                    my ($twig, $comment) = @_;

                    my ($row, $col) = $self->_cell_to_row_col($comment->att('ref'));
                    my $text = $comment->text;
                    my $cell = $sheet->{Cells}[$row][$col];
                    $cell->{_comment} = $text;
                },
            },
        );
        $comment_xml->parse($comment_contents);
    }
}

1;

__END__

=pod

=head1 NAME

Spreadsheet::ParseXLSX::ChopRPH

=head1 VERSION

version 0.201409142

=head1 AUTHOR

Tomohiro Hosaka <bokutin@bokut.in>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2014 by Tomohiro Hosaka.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
